@@ -6,30 +6,40 @@ chromosome for genetic algorithm
 from __future__ import annotations
 import logging
 import numpy as np
-from typing import Union, List, Tuple
+from typing import Dict, Union, List, Tuple
 from .utils import limit, GENE_MIN, GENE_MAX
 
 logger = logging.getLogger('openGA')
 
+GENE_PRECISION = 4
+
+
+def dist_crossover(r: float, eta: Union[int, float] = 20) -> float:
+    k = 1 / (eta + 1)
+    if r <= 0.5:
+        rlt = (2 * r) ** k
+    else:
+        rlt = (2 * (1 - r)) ** -k
+    return rlt
+
+
+def dist_mutation(r: float, eta: Union[int, float] = 20) -> float:
+    k = 1 / (eta + 1)
+    if r < 0.5:
+        rlt = (2 * r) ** k - 1
+    else:
+        rlt = 1 - (2 * (1 - r)) ** k
+    return rlt
+
 
 def get_crossover_coef(eta: Union[int, float] = 20) -> float:
     u = np.random.uniform()
-    k = 1 / (eta + 1)
-    if u <= 0.5:
-        rlt = (2 * u) ** k
-    else:
-        rlt = (2 * (1 - u)) ** -k
-    return rlt
+    return dist_crossover(u, eta)
 
 
 def get_mutation_coef(eta: Union[int, float] = 20) -> float:
     u = np.random.uniform()
-    k = 1 / (eta + 1)
-    if u < 0.5:
-        rlt = (2 * u) ** k - 1
-    else:
-        rlt = 1 - (2 * (1 - u)) ** k
-    return rlt
+    return dist_mutation(u, eta)
 
 
 class Chromosome(object):
@@ -43,6 +53,32 @@ class Chromosome(object):
         self._gene_names = gene_names.copy()
         self._gene_values = np.zeros(self._gene_num)
 
+    def to_dict(self) -> Dict[str, float]:
+        rlt = {}
+        for i in range(self._gene_num):
+            rlt[self._gene_names[i]] = self._gene_values[i]
+        return rlt
+
+    def __str__(self) -> str:
+        return str(self.to_dict())
+
+    def __format__(self, format_spec: str) -> str:
+        return str(self)
+
+    def __eq__(self, o: Chromosome) -> bool:
+        if self._gene_num != o._gene_num:
+            return False
+        if self._check != o._check:
+            return False
+        if self._gene_names != o._gene_names:
+            return False
+        if np.any(self._gene_values != o._gene_values):
+            return False
+        return True
+
+    def size(self):
+        return self._gene_num
+
     @property
     def check(self):
         return self._check
@@ -55,14 +91,14 @@ class Chromosome(object):
     def gene_values(self):
         return self._gene_values.copy()
 
-    def _update(self, value: float, index: int):
+    def update(self, value: float, index: int):
         if not GENE_MIN <= value <= GENE_MAX and self._check:
             value_set = limit(value)
             logger.warning(
                 'get out of boundary value [%6.4f], limit to [%6.4f]' % (value, value_set))
         else:
             value_set = value
-        self._gene_values[index] = value_set
+        self._gene_values[index] = round(value_set, GENE_PRECISION)
 
     def is_couple(self, couple: Chromosome) -> bool:
         if self._gene_num != couple._gene_num:
@@ -77,7 +113,9 @@ class Chromosome(object):
 
     def random(self):
         for i in range(self._gene_num):
-            self._update(np.random.uniform(GENE_MIN, GENE_MAX), i)
+            val = np.random.uniform(GENE_MIN, GENE_MAX)
+            val = round(val, GENE_PRECISION)
+            self.update(val, i)
 
     def crossover(self, couple: Chromosome, eta: Union[int, float] = 20) -> Tuple[Chromosome, Chromosome]:
         if self._check:
@@ -93,8 +131,8 @@ class Chromosome(object):
             b = 1 + beta
             c0_gene_val = limit(0.5 * (a * p0_gene_val + b * p1_gene_val))
             c1_gene_val = limit(0.5 * (b * p0_gene_val + a * p1_gene_val))
-            offspring_0._update(c0_gene_val, i)
-            offspring_1._update(c1_gene_val, i)
+            offspring_0.update(c0_gene_val, i)
+            offspring_1.update(c1_gene_val, i)
         return offspring_0, offspring_1
 
     def mutate(self, eta: Union[int, float] = 20) -> Chromosome:
@@ -103,11 +141,11 @@ class Chromosome(object):
             p_gene_val = self._gene_values[i]
             theta = get_mutation_coef(eta)
             c_gene_val = limit(p_gene_val + theta)
-            offspring._update(c_gene_val, i)
+            offspring.update(c_gene_val, i)
         return offspring
 
     def copy(self) -> Chromosome:
         twin = Chromosome(self._gene_names, self._check)
         for i in range(self._gene_num):
-            twin._update(self._gene_values[i], i)
+            twin.update(self._gene_values[i], i)
         return twin
